# -*- coding: utf-8 -*-
"""
Created on Wed Dec 19 15:50:41 2018

@author: Andreas
"""
from nion.utils import Binding
from nion.swift import MimeTypes
from nion.swift import DataItemThumbnailWidget
from nion.utils import Geometry
import uuid
import copy

def make_image_chooser(document_controller, computation, variable):
    ui = document_controller.ui
    document_model = document_controller.document_model
    column = ui.create_column_widget()
    row = ui.create_row_widget()
    label_column = ui.create_column_widget()
    label_widget = ui.create_label_widget(variable.display_label, properties={"width": 80})
    label_widget.bind_text(Binding.PropertyBinding(variable, "display_label"))
    label_column.add(label_widget)
    label_column.add_stretch()
    row.add(label_column)
    row.add_spacing(8)

    def drop_mime_data(mime_data, x, y):
        if mime_data.has_format(MimeTypes.DISPLAY_ITEM_MIME_TYPE):
            display_item_uuid = uuid.UUID(mime_data.data_as_string(MimeTypes.DISPLAY_ITEM_MIME_TYPE))
            display_item = document_model.get_display_item_by_uuid(display_item_uuid)
            data_item = display_item.data_item if display_item else None
            if data_item:
                variable_specifier = document_model.get_object_specifier(display_item.display_data_channel)
                if variable.objects_model.items:
                    variable.objects_model.remove_item(0)
                variable.objects_model.append_item(variable_specifier)
                return "copy"
        return None

    def data_item_delete():
        if variable.objects_model.items:
            variable.objects_model.remove_item(0)

    display_item = None
    if variable.object_specifiers:
        base_variable_specifier = copy.copy(variable.object_specifiers[0])
        bound_data_source = document_model.resolve_object_specifier(base_variable_specifier)
        data_item = bound_data_source.value.data_item if bound_data_source else None
        display_item = document_model.get_display_item_for_data_item(data_item)
    data_item_thumbnail_source = DataItemThumbnailWidget.DataItemThumbnailSource(ui, display_item=display_item)
    data_item_chooser_widget = DataItemThumbnailWidget.ThumbnailWidget(ui, data_item_thumbnail_source, Geometry.IntSize(80, 80))

    def thumbnail_widget_drag(mime_data, thumbnail, hot_spot_x, hot_spot_y):
        # use this convoluted base object for drag so that it doesn't disappear after the drag.
        try:
            column.drag(mime_data, thumbnail, hot_spot_x, hot_spot_y)
        except Exception as e:
            print(e)
            import traceback
            traceback.print_exc()

    data_item_chooser_widget.on_drag = thumbnail_widget_drag
    data_item_chooser_widget.on_drop_mime_data = drop_mime_data
    data_item_chooser_widget.on_delete = data_item_delete

    def property_changed(key):
        if key == "object_specifiers":
            if variable.object_specifiers:
                base_variable_specifier = copy.copy(variable.object_specifiers[0])
                bound_data_item = document_model.resolve_object_specifier(base_variable_specifier)
                data_item = bound_data_item.value.data_item if bound_data_item else None
                display_item = document_model.get_display_item_for_data_item(data_item)
                data_item_thumbnail_source.set_display_item(display_item)
            else:
                data_item_thumbnail_source.set_display_item(None)

    property_changed_listener = variable.property_changed_event.listen(property_changed)
    row.add(data_item_chooser_widget)
    row.add_stretch()
    column.add(row)
    column.add_spacing(4)
    return column, [property_changed_listener]



# -*- coding: utf-8 -*-
"""
Created on Mon Dec 17 18:17:31 2018

@author: Andreas
"""

class ComputationUIPanelDelegate(object):

    def __init__(self, api):
        self.__api = api
        self.panel_id = 'ComputationUI-Panel'
        self.panel_name = 'Computation'
        self.panel_positions = ['left', 'right']
        self.panel_position = 'right'
        self.api = api

    def create_panel_widget(self, ui, document_controller):
        self.ui = ui
        self.document_controller = document_controller
        self.__display_item_changed_event_listener = (
                                 document_controller._document_controller.focused_display_item_changed_event.listen(
                                                                                         self.__display_item_changed))
        self.__computation_updated_event_listeners = []

        self.column = ui.create_column_widget()
        self.column.add(self.ui.create_label_widget('test'))

        return self.column

    def __update_computation_ui(self, computations):
        self.column._widget.remove_all()
        for computation in computations:
            create_panel_widget = getattr(computation, 'create_panel_widget', None)
            if create_panel_widget:
                try:
                    widget = create_panel_widget(self.ui, self.document_controller)
                except Exception as e:
                    print(str(e))
                    import traceback
                    traceback.print_exc()
                else:
                    self.column.add(self.ui.create_label_widget(computation.processing_id))
                    self.column.add_spacing(2)
                    self.column.add(widget)
                    self.column.add_spacing(5)
        self.column.add_stretch()


    def __display_item_changed(self, display_item):
        data_item = display_item.data_item if display_item else None
        if data_item:
            for listener in self.__computation_updated_event_listeners:
                listener.close()
            self.__computation_updated_event_listeners = []
            computations = self.document_controller._document_controller.document_model.computations
            computations_involved = []
            for computation in computations:
                for result in computation.results:
                    if result.specifier.get('uuid') == str(data_item.uuid):
                        computations_involved.append(computation)
                if computation.source:
                    if str(computation.source.uuid) == str(data_item.uuid):
                        computations_involved.append(computation)

            self.__update_computation_ui(computations_involved)
            for _ in computations_involved:
                self.__computation_updated_event_listeners.append(
                        self.document_controller._document_controller.document_model.computation_updated_event.listen(
                                lambda data_item, new_computation: self.__update_computation_ui(computations_involved)))
            if not computations_involved:
                self.column._widget.remove_all()

    def close(self):
        self.__display_item_changed_event_listener.close()
        self.__display_item_changed_event_listener = None

class ComputationUIExtension(object):
    extension_id = 'nion.extension.computation_ui'

    def __init__(self, api_broker):
        api = api_broker.get_api(version='1', ui_version='1')
        self.__panel_ref = api.create_panel(ComputationUIPanelDelegate(api))

    def close(self):
        self.__panel_ref.close()
        self.__panel_ref = None



# standard libraries
import gettext

# third party libraries
# none

# local libraries
from nion.typeshed import API_1_0 as API
from nion.swift.model import DataItem
from nion.swift.model import DocumentModel

_ = gettext.gettext


double_gaussian_script = """
# imports
import math
import numpy
import scipy

# get the data
data = src.data

# first calculate the FFT
fft_data = scipy.fftpack.fftshift(scipy.fftpack.fft2(data))

# next, set up xx, yy arrays to be linear indexes for x and y coordinates ranging
# from -width/2 to width/2 and -height/2 to height/2.
yy_min = int(math.floor(-data.shape[0] / 2))
yy_max = int(math.floor(data.shape[0] / 2))
xx_min = int(math.floor(-data.shape[1] / 2))
xx_max = int(math.floor(data.shape[1] / 2))
xx, yy = numpy.meshgrid(numpy.linspace(yy_min, yy_max, data.shape[0]),
                        numpy.linspace(xx_min, xx_max, data.shape[1]))

# calculate the pixel distance from the center
rr = numpy.sqrt(numpy.square(xx) + numpy.square(yy)) / (data.shape[0] * 0.5)

# finally, apply a filter to the Fourier space data.
filter = numpy.exp(-0.5 * numpy.square(rr / sigma1)) - (1.0 - weight2) * numpy.exp(
    -0.5 * numpy.square(rr / sigma2))
filtered_fft_data = fft_data * filter

# and then do invert FFT and take the real value.
result = scipy.fftpack.ifft2(scipy.fftpack.ifftshift(filtered_fft_data)).real

intensity_calibration = src.xdata.intensity_calibration
dimensional_calibrations = src.xdata.dimensional_calibrations
target.xdata = api.create_data_and_metadata_from_data(result, intensity_calibration, dimensional_calibrations)
"""


processing_descriptions = {
    "nion.extension.doublegaussian":
        { 'script': double_gaussian_script,
          'sources': [
              {"name": "src", "label": _("Source"), "requirements": [{"type": "dimensionality", "min": 2, "max": 2}]}
          ],
          "parameters": [
              {"name": "sigma1", "label": _("Sigma 1"), "type": "real", "value": 0.3, "value_default": 0.3, "value_min": 0, "value_max": 1, "control_type": "slider"},
              {"name": "sigma2", "label": _("Sigma 2"), "type": "real", "value": 0.3, "value_default": 0.3, "value_min": 0, "value_max": 1, "control_type": "slider"},
              {"name": "weight2", "label": _("Weight 2"), "type": "real", "value": 0.3, "value_default": 0.3, "value_min": 0, "value_max": 1, "control_type": "slider"},
          ],
          'title': 'Double Gaussian',
        }
}


DocumentModel.DocumentModel.register_processing_descriptions(processing_descriptions)


class DoubleGaussianMenuItem:

    menu_id = "_processing_menu"  # required, specify menu_id where this item will go
    menu_item_name = _("Double Gaussian")  # menu item name

    def menu_item_execute(self, window: API.DocumentWindow) -> None:
        document_controller = window._document_controller
        selected_display_item = document_controller.selected_display_item
        data_item = document_controller.document_model.make_data_item_with_computation("nion.extension.doublegaussian", [(selected_display_item, None)])
        if data_item:
            display_item = document_controller.document_model.get_display_item_for_data_item(data_item)
            document_controller.show_display_item(display_item)


class DoubleGaussianExtension:

    # required for Swift to recognize this as an extension class.
    extension_id = "nionswift.extension.double_gaussian"

    def __init__(self, api_broker):
        # grab the api object.
        api = api_broker.get_api(version="1", ui_version="1")
        # be sure to keep a reference or it will be closed immediately.
        self.__menu_item_ref = api.create_menu_item(DoubleGaussianMenuItem())

    def close(self):
        self.__menu_item_ref.close()



# -*- coding: utf-8 -*-

"""
To upload to PyPI, PyPI test, or a local server:
python setup.py bdist_wheel upload -r <server_identifier>
"""

import setuptools
import os

setuptools.setup(
    name="nionswift-experimental",
    version="0.6.0",
    author="Nion Software",
    author_email="swift@nion.com",
    description="Experimental tools package for Nion Swift.",
    packages=["nionswift_plugin.nion_experimental_tools", "nionswift_plugin.nion_experimental_4dtools"],
    install_requires=["nionswift>=0.14.0"],
    license='GPLv3',
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
    ],
    include_package_data=True,
    python_requires='~=3.6',
    zip_safe=False,
)



# system imports
import gettext

# local libraries
from nion.typeshed import API_1_0 as API

_ = gettext.gettext


class AlignToCenterMenuItem:

    menu_id = "_edit_menu"  # required, specify menu_id where this item will go
    menu_item_name = _("Align Graphic to Center")  # menu item name

    def menu_item_execute(self, window: API.DocumentWindow) -> None:
        target_display = window.target_display
        if target_display:
            _display_item = target_display._display_item
            if _display_item.graphic_selection.has_selection:
                graphics = [_display_item.graphics[index] for index in _display_item.graphic_selection.indexes]
                for graphic in graphics:
                    if hasattr(graphic, "position"):
                        graphic.position = (0.5, 0.5)
                    if hasattr(graphic, "center"):
                        graphic.center = (0.5, 0.5)


class GraphicsToolsExtension:

    # required for Swift to recognize this as an extension class.
    extension_id = "nionswift.graphics_tools"

    def __init__(self, api_broker):
        # grab the api object.
        api = api_broker.get_api(version="1", ui_version="1")
        # be sure to keep a reference or it will be closed immediately.
        self.__menu_item_ref = api.create_menu_item(AlignToCenterMenuItem())

    def close(self):
        self.__menu_item_ref.close()
